@@ -42,12 +42,14 @@
 #include "config.h"
 #endif
 
-#include "clutter-backend.h"
+#include "clutter-backend-private.h"
 #include "clutter-debug.h"
 #include "clutter-fixed.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-profile.h"
+#include "clutter-stage-manager-private.h"
+#include "clutter-stage-private.h"
 
 #include <cogl/cogl.h>
 
@@ -66,6 +68,8 @@ struct _ClutterBackendPrivate
 
   gfloat units_per_em;
   gint32 units_serial;
+
+  GList *event_translators;
 };
 
 enum
@@ -82,17 +86,16 @@ static guint backend_signals[LAST_SIGNAL] = { 0, };
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterMainContext *clutter_context;
+  ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
 
-  clutter_context = _clutter_context_get_default ();
+  /* clear the events still in the queue of the main context */
+  _clutter_clear_events_queue ();
 
-  if (clutter_context && clutter_context->events_queue)
+  /* remove all event translators */
+  if (priv->event_translators != NULL)
     {
-      g_queue_foreach (clutter_context->events_queue,
-                       (GFunc) clutter_event_free,
-                       NULL);
-      g_queue_free (clutter_context->events_queue);
-      clutter_context->events_queue = NULL;
+      g_list_free (priv->event_translators);
+      priv->event_translators = NULL;
     }
 
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
@@ -202,6 +205,51 @@ clutter_backend_real_font_changed (ClutterBackend *backend)
   CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
+static void
+clutter_backend_real_redraw (ClutterBackend *backend,
+                             ClutterStage   *stage)
+{
+  ClutterStageWindow *impl;
+
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
+    return;
+
+  impl = _clutter_stage_get_window (stage);
+  if (impl == NULL)
+    return;
+
+  _clutter_stage_window_redraw (impl);
+}
+
+static gboolean
+clutter_backend_real_translate_event (ClutterBackend *backend,
+                                      gpointer        native,
+                                      ClutterEvent   *event)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+  GList *l;
+
+  for (l = priv->event_translators;
+       l != NULL;
+       l = l->next)
+    {
+      ClutterEventTranslator *translator = l->data;
+      ClutterTranslateReturn retval;
+
+      retval = _clutter_event_translator_translate_event (translator,
+                                                          native,
+                                                          event);
+
+      if (retval == CLUTTER_TRANSLATE_QUEUE)
+        return TRUE;
+
+      if (retval == CLUTTER_TRANSLATE_REMOVE)
+        return FALSE;
+    }
+
+  return FALSE;
+}
+
 static void
 clutter_backend_class_init (ClutterBackendClass *klass)
 {
@@ -241,6 +289,8 @@ clutter_backend_class_init (ClutterBackendClass *klass)
 
   klass->resolution_changed = clutter_backend_real_resolution_changed;
   klass->font_changed = clutter_backend_real_font_changed;
+  klass->translate_event = clutter_backend_real_translate_event;
+  klass->redraw = clutter_backend_real_redraw;
 }
 
 static void
@@ -331,7 +381,6 @@ void
 _clutter_backend_redraw (ClutterBackend *backend,
                          ClutterStage   *stage)
 {
-  ClutterBackendClass *klass;
   CLUTTER_STATIC_COUNTER (redraw_counter,
                           "_clutter_backend_redraw counter",
                           "Increments for each _clutter_backend_redraw call",
@@ -345,9 +394,7 @@ _clutter_backend_redraw (ClutterBackend *backend,
   CLUTTER_COUNTER_INC (_clutter_uprof_context, redraw_counter);
   CLUTTER_TIMER_START (_clutter_uprof_context, redraw_timer);
 
-  klass = CLUTTER_BACKEND_GET_CLASS (backend);
-  if (G_LIKELY (klass->redraw))
-    klass->redraw (backend, stage);
+  CLUTTER_BACKEND_GET_CLASS (backend)->redraw (backend, stage);
 
   CLUTTER_TIMER_STOP (_clutter_uprof_context, redraw_timer);
 }
@@ -497,8 +544,6 @@ _clutter_backend_init_events (ClutterBackend *backend)
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
   g_return_if_fail (clutter_context != NULL);
 
-  clutter_context->events_queue = g_queue_new ();
-
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->init_events)
     klass->init_events (backend);
@@ -879,3 +924,41 @@ _clutter_backend_get_units_serial (ClutterBackend *backend)
 
   return backend->priv->units_serial;
 }
+
+gboolean
+_clutter_backend_translate_event (ClutterBackend *backend,
+                                  gpointer        native,
+                                  ClutterEvent   *event)
+{
+  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
+
+  return CLUTTER_BACKEND_GET_CLASS (backend)->translate_event (backend,
+                                                               native,
+                                                               event);
+}
+
+void
+_clutter_backend_add_event_translator (ClutterBackend         *backend,
+                                       ClutterEventTranslator *translator)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) != NULL)
+    return;
+
+  priv->event_translators =
+    g_list_prepend (priv->event_translators, translator);
+}
+
+void
+_clutter_backend_remove_event_translator (ClutterBackend         *backend,
+                                          ClutterEventTranslator *translator)
+{
+  ClutterBackendPrivate *priv = backend->priv;
+
+  if (g_list_find (priv->event_translators, translator) == NULL)
+    return;
+
+  priv->event_translators =
+    g_list_remove (priv->event_translators, translator);
+}
