@@ -1163,34 +1163,20 @@ clutter_stage_dispose (GObject *object)
 {
   ClutterStage        *stage = CLUTTER_STAGE (object);
   ClutterStagePrivate *priv = stage->priv;
-  ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
-  ClutterMainContext  *context;
-  GList               *l, *next;
+  ClutterStageManager *stage_manager;
 
   clutter_actor_hide (CLUTTER_ACTOR (object));
 
+  stage_manager = clutter_stage_manager_get_default ();
   _clutter_stage_manager_remove_stage (stage_manager, stage);
 
-  context = _clutter_context_get_default ();
-
-  /* Remove any pending events for this stage from the event queue */
-  for (l = context->events_queue->head; l; l = next)
-    {
-      ClutterEvent *event = l->data;
-
-      next = l->next;
-
-      if (event->any.stage == stage)
-        {
-          g_queue_delete_link (context->events_queue, l);
-          clutter_event_free (event);
-        }
-    }
+  _clutter_clear_events_queue_for_stage (stage);
 
   if (priv->impl != NULL)
     {
       CLUTTER_NOTE (BACKEND, "Disposing of the stage implementation");
 
+      _clutter_stage_window_unrealize (priv->impl);
       g_object_unref (priv->impl);
       priv->impl = NULL;
     }
@@ -1204,7 +1190,7 @@ clutter_stage_finalize (GObject *object)
   ClutterStage *stage = CLUTTER_STAGE (object);
   ClutterStagePrivate *priv = stage->priv;
 
-  g_queue_foreach (priv->event_queue, (GFunc)clutter_event_free, NULL);
+  g_queue_foreach (priv->event_queue, (GFunc) clutter_event_free, NULL);
   g_queue_free (priv->event_queue);
 
   g_free (priv->title);
