@@ -40,15 +40,22 @@
 #include "clutter-stage-egl.h"
 #include "clutter-egl.h"
 
-#include "../clutter-private.h"
-#include "../clutter-main.h"
-#include "../clutter-debug.h"
-#include "../clutter-version.h"
+#ifdef HAVE_EVDEV
+#include "clutter-device-manager-evdev.h"
+#endif
+
+#include "clutter-debug.h"
+#include "clutter-private.h"
+#include "clutter-main.h"
+#include "clutter-stage-private.h"
+#include "clutter-version.h"
 
 static ClutterBackendEGL *backend_singleton = NULL;
 
 static const gchar *clutter_fb_device = NULL;
 
+static gchar *clutter_vblank_name = NULL;
+
 #ifdef COGL_HAS_X11_SUPPORT
 G_DEFINE_TYPE (ClutterBackendEGL, _clutter_backend_egl, CLUTTER_TYPE_BACKEND_X11);
 #else
@@ -75,6 +82,13 @@ clutter_backend_egl_pre_parse (ClutterBackend  *backend,
     return FALSE;
 #endif
 
+  env_string = g_getenv ("CLUTTER_VBLANK");
+  if (env_string)
+    {
+      clutter_vblank_name = g_strdup (env_string);
+      env_string = NULL;
+    }
+
   env_string = g_getenv ("CLUTTER_FB_DEVICE");
   if (env_string != NULL && env_string[0] != '\0')
     clutter_fb_device = g_strdup (env_string);
@@ -132,6 +146,34 @@ clutter_backend_egl_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
+void
+_clutter_backend_egl_blit_sub_buffer (ClutterBackendEGL *backend_egl,
+                                      EGLSurface drawable,
+                                      int x, int y, int width, int height)
+{
+  if (backend_egl->swap_buffers_region)
+    {
+      EGLint rect[4] = {
+          x, y, width, height
+      };
+      backend_egl->swap_buffers_region (backend_egl->edpy,
+                                        drawable,
+                                        1,
+                                        rect);
+    }
+#if 0 /* XXX need GL_ARB_draw_buffers */
+  else if (backend_egl->blit_framebuffer)
+    {
+      glDrawBuffer (GL_FRONT);
+      backend_egl->blit_framebuffer (x, y, x + width, y + height,
+                                     x, y, x + width, y + height,
+                                     GL_COLOR_BUFFER_BIT, GL_NEAREST);
+      glDrawBuffer (GL_BACK);
+      glFlush();
+    }
+#endif
+}
+
 static gboolean
 clutter_backend_egl_create_context (ClutterBackend  *backend,
                                     GError         **error)
@@ -180,7 +222,7 @@ clutter_backend_egl_create_context (ClutterBackend  *backend,
   if (backend_egl->egl_context != EGL_NO_CONTEXT)
     return TRUE;
 
-  edpy = clutter_egl_display ();
+  edpy = clutter_egl_get_egl_display ();
 
 /* XXX: we should get rid of this goto yukkyness, there is a fail:
  * goto at the end and this retry: goto at the top, but we should just
@@ -248,7 +290,7 @@ retry:
    * a dummy, offscreen override-redirect window to which we can always
    * fall back if no stage is available */
 
-  xvisinfo = clutter_backend_x11_get_visual_info (backend_x11);
+  xvisinfo = _clutter_backend_x11_get_visual_info (backend_x11);
   if (xvisinfo == NULL)
     {
       g_critical ("Unable to find suitable GL visual.");
@@ -463,29 +505,42 @@ clutter_backend_egl_ensure_context (ClutterBackend *backend,
 #endif /* COGL_HAS_XLIB_SUPPORT */
 }
 
-static void
-clutter_backend_egl_redraw (ClutterBackend *backend,
-                            ClutterStage   *stage)
+#ifndef COGL_HAS_XLIB_SUPPORT
+static ClutterDeviceManager *
+clutter_backend_egl_get_device_manager (ClutterBackend *backend)
 {
-  ClutterStageWindow *impl;
+  ClutterBackendEGL *backend_egl = CLUTTER_BACKEND_EGL (backend);
 
-  impl = _clutter_stage_get_window (stage);
-  if (!impl)
-    return;
+  if (G_UNLIKELY (backend_egl->device_manager == NULL))
+    {
+#ifdef HAVE_EVDEV
+      backend_egl->device_manager =
+	g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_EVDEV,
+		      "backend", backend_egl,
+		      NULL);
+#endif
+    }
 
-  g_assert (CLUTTER_IS_STAGE_EGL (impl));
-
-  _clutter_stage_egl_redraw (CLUTTER_STAGE_EGL (impl), stage);
+  return backend_egl->device_manager;
 }
+#endif
 
-#ifdef HAVE_TSLIB
 static void
 clutter_backend_egl_init_events (ClutterBackend *backend)
 {
+#ifdef HAVE_TSLIB
   /* XXX: This should be renamed to _clutter_events_tslib_init */
   _clutter_events_egl_init (CLUTTER_BACKEND_EGL (backend));
-}
 #endif
+#ifdef HAVE_EVDEV
+  _clutter_events_evdev_init (CLUTTER_BACKEND (backend));
+#endif
+#ifdef COGL_HAS_X11_SUPPORT
+  /* Chain up to the X11 backend */
+  CLUTTER_BACKEND_CLASS (_clutter_backend_egl_parent_class)->
+    init_events (backend);
+#endif
+}
 
 static void
 clutter_backend_egl_finalize (GObject *gobject)
@@ -597,21 +652,70 @@ clutter_backend_egl_constructor (GType                  gtype,
   return g_object_ref (backend_singleton);
 }
 
+static gboolean
+check_vblank_env (const char *name)
+{
+  if (clutter_vblank_name && !g_ascii_strcasecmp (clutter_vblank_name, name))
+    return TRUE;
+
+  return FALSE;
+}
+
 static ClutterFeatureFlags
 clutter_backend_egl_get_features (ClutterBackend *backend)
 {
-  ClutterBackendEGL  *backend_egl = CLUTTER_BACKEND_EGL (backend);
+  ClutterBackendEGL *backend_egl = CLUTTER_BACKEND_EGL (backend);
+  const gchar *egl_extensions = NULL;
+  const gchar *gl_extensions = NULL;
   ClutterFeatureFlags flags;
 
   g_assert (backend_egl->egl_context != NULL);
 
 #ifdef COGL_HAS_XLIB_SUPPORT
-  flags = clutter_backend_x11_get_features (backend);
-  flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
+  {
+    ClutterBackendClass *parent_class;
+
+    parent_class = CLUTTER_BACKEND_CLASS (_clutter_backend_egl_parent_class);
+    flags = parent_class->get_features (backend);
+    flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
+  }
 #else
   flags = CLUTTER_FEATURE_STAGE_STATIC;
 #endif
 
+  /* First check for explicit disabling or it set elsewhere (eg NVIDIA) */
+  if (check_vblank_env ("none"))
+    backend_egl->vblank_type = CLUTTER_VBLANK_NONE;
+  else
+    backend_egl->vblank_type = CLUTTER_VBLANK_SWAP_INTERVAL;
+
+  egl_extensions = eglQueryString (backend_egl->edpy, EGL_EXTENSIONS);
+
+  if (_cogl_check_extension ("EGL_NOK_swap_region", egl_extensions))
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Using EGL_NOK_swap_region for sub_buffer copies");
+      backend_egl->swap_buffers_region =
+        (void *)cogl_get_proc_address ("eglSwapBuffersRegionNOK");
+      backend_egl->can_blit_sub_buffer = TRUE;
+      backend_egl->blit_sub_buffer_is_synchronized = TRUE;
+    }
+
+  gl_extensions = (const gchar *)glGetString (GL_EXTENSIONS);
+
+#if 0 /* XXX need GL_ARB_draw_buffers */
+  if (!backend_egl->swap_buffers_region &&
+      _cogl_check_extension ("GL_EXT_framebuffer_blit", gl_extensions))
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Using glBlitFramebuffer fallback for sub_buffer copies");
+      backend_egl->blit_framebuffer =
+        (BlitFramebufferProc) cogl_get_proc_address ("glBlitFramebuffer");
+      backend_egl->can_blit_sub_buffer = TRUE;
+      backend_egl->blit_sub_buffer_is_synchronized = FALSE;
+    }
+#endif
+
   CLUTTER_NOTE (BACKEND, "Checking features\n"
                 "GL_VENDOR: %s\n"
                 "GL_RENDERER: %s\n"
@@ -636,6 +740,7 @@ clutter_backend_egl_create_stage (ClutterBackend  *backend,
 {
 #ifdef COGL_HAS_XLIB_SUPPORT
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterEventTranslator *translator;
   ClutterStageWindow *stage;
   ClutterStageX11 *stage_x11;
 
@@ -647,6 +752,10 @@ clutter_backend_egl_create_stage (ClutterBackend  *backend,
   /* copy backend data into the stage */
   stage_x11 = CLUTTER_STAGE_X11 (stage);
   stage_x11->wrapper = wrapper;
+  stage_x11->backend = backend_x11;
+
+  translator = CLUTTER_EVENT_TRANSLATOR (stage_x11);
+  _clutter_backend_add_event_translator (backend, translator);
 
   CLUTTER_NOTE (MISC, "EGLX stage created (display:%p, screen:%d, root:%u)",
                 backend_x11->xdpy,
@@ -745,16 +854,16 @@ _clutter_backend_egl_class_init (ClutterBackendEGLClass *klass)
   gobject_class->dispose     = clutter_backend_egl_dispose;
   gobject_class->finalize    = clutter_backend_egl_finalize;
 
-  backend_class->pre_parse        = clutter_backend_egl_pre_parse;
-  backend_class->post_parse       = clutter_backend_egl_post_parse;
-  backend_class->get_features     = clutter_backend_egl_get_features;
-#ifdef HAVE_TSLIB
-  backend_class->init_events      = clutter_backend_egl_init_events;
+  backend_class->pre_parse          = clutter_backend_egl_pre_parse;
+  backend_class->post_parse         = clutter_backend_egl_post_parse;
+  backend_class->get_features       = clutter_backend_egl_get_features;
+#ifndef COGL_HAS_XLIB_SUPPORT
+  backend_class->get_device_manager = clutter_backend_egl_get_device_manager;
 #endif
-  backend_class->create_stage     = clutter_backend_egl_create_stage;
-  backend_class->create_context   = clutter_backend_egl_create_context;
-  backend_class->ensure_context   = clutter_backend_egl_ensure_context;
-  backend_class->redraw           = clutter_backend_egl_redraw;
+  backend_class->init_events        = clutter_backend_egl_init_events;
+  backend_class->create_stage       = clutter_backend_egl_create_stage;
+  backend_class->create_context     = clutter_backend_egl_create_context;
+  backend_class->ensure_context     = clutter_backend_egl_ensure_context;
 
 #ifdef COGL_HAS_XLIB_SUPPORT
   backendx11_class->get_visual_info = clutter_backend_egl_get_visual_info;
@@ -788,17 +897,26 @@ _clutter_backend_impl_get_type (void)
 }
 #endif
 
-#ifdef COGL_HAS_XLIB_SUPPORT
 EGLDisplay
 clutter_eglx_display (void)
 {
-  return backend_singleton->edpy;
+  return clutter_egl_get_egl_display ();
 }
-#endif /* COGL_HAS_XLIB_SUPPORT */
 
 EGLDisplay
 clutter_egl_display (void)
 {
-  return backend_singleton->edpy;
+  return clutter_egl_get_egl_display ();
 }
 
+EGLDisplay
+clutter_egl_get_egl_display (void)
+{
+  if (backend_singleton == NULL)
+    {
+      g_critical ("%s has been called before clutter_init()", G_STRFUNC);
+      return 0;
+    }
+
+  return backend_singleton->edpy;
+}
