@@ -525,8 +525,14 @@ _clutter_backend_glx_blit_sub_buffer (ClutterBackendGLX *backend_glx,
                                      x, y, x + width, y + height,
                                      GL_COLOR_BUFFER_BIT, GL_NEAREST);
       glDrawBuffer (GL_BACK);
-      glFlush();
     }
+
+  /* NB: unlike glXSwapBuffers, glXCopySubBuffer and
+   * glBlitFramebuffer don't issue an implicit glFlush() so we
+   * have to flush ourselves if we want the request to complete in
+   * finite amount of time since otherwise the driver can batch
+   * the command indefinitely. */
+  glFlush();
 }
 
 static XVisualInfo *
@@ -772,28 +778,6 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
     }
 }
 
-/*
- * FIXME: we should remove backend_class->redraw() and just
- * have stage_window_iface->redraw()
- */
-static void
-clutter_backend_glx_redraw (ClutterBackend *backend,
-                            ClutterStage   *stage)
-{
-  ClutterStageWindow *impl = _clutter_stage_get_window (stage);
-
-  if (G_UNLIKELY (impl == NULL))
-    {
-      CLUTTER_NOTE (BACKEND, "Stage [%p] has no implementation", stage);
-      return;
-    }
-
-  g_assert (CLUTTER_IS_STAGE_GLX (impl));
-
-  _clutter_stage_glx_redraw (CLUTTER_STAGE_GLX (impl),
-                             stage);
-}
-
 static ClutterStageWindow *
 clutter_backend_glx_create_stage (ClutterBackend  *backend,
                                   ClutterStage    *wrapper,
@@ -815,7 +799,7 @@ clutter_backend_glx_create_stage (ClutterBackend  *backend,
   stage_x11->backend = backend_x11;
 
   translator = CLUTTER_EVENT_TRANSLATOR (stage_x11);
-  _clutter_backend_x11_add_event_translator (backend_x11, translator);
+  _clutter_backend_add_event_translator (backend, translator);
 
   CLUTTER_NOTE (BACKEND,
                 "GLX stage created[%p] (dpy:%p, screen:%d, root:%u, wrap:%p)",
@@ -844,7 +828,6 @@ clutter_backend_glx_class_init (ClutterBackendGLXClass *klass)
   backend_class->create_stage   = clutter_backend_glx_create_stage;
   backend_class->add_options    = clutter_backend_glx_add_options;
   backend_class->get_features   = clutter_backend_glx_get_features;
-  backend_class->redraw         = clutter_backend_glx_redraw;
   backend_class->create_context = clutter_backend_glx_create_context;
   backend_class->ensure_context = clutter_backend_glx_ensure_context;
 
