@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2007-2008  Tommi Komulainen <tommi.komulainen@iki.fi>
  * Copyright (C) 2007  OpenedHand Ltd.
+ * Copyright (C) 2011  Crystalnix  <vgachkaylo@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,6 +34,8 @@
 
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
+#define clutter_stage_osx_get_type      _clutter_stage_osx_get_type
+
 G_DEFINE_TYPE_WITH_CODE (ClutterStageOSX,
                          clutter_stage_osx,
                          G_TYPE_OBJECT,
@@ -44,6 +47,7 @@ static void
 clutter_stage_osx_state_update (ClutterStageOSX   *self,
                                 ClutterStageState  unset_flags,
                                 ClutterStageState  set_flags);
+
 static ClutterActor *
 clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
 
@@ -132,6 +136,7 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
 @interface ClutterGLView : NSOpenGLView
 {
   ClutterStageOSX *stage_osx;
+  NSTrackingRectTag tracking_rect;
 }
 - (void) drawRect: (NSRect) bounds;
 @end
@@ -142,6 +147,7 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
   if ((self = [super initWithFrame:aFrame pixelFormat:aFormat]) != nil)
     {
       self->stage_osx = aStage;
+      tracking_rect = [self addTrackingRect:[self bounds] owner:self userData:NULL assumeInside:NO];
     }
 
   return self;
@@ -173,6 +179,9 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
   stage_osx->requisition_height = [self bounds].size.height;
   clutter_actor_set_size (CLUTTER_ACTOR (self->stage_osx->wrapper),
                           (int)[self bounds].size.width, (int)[self bounds].size.height);
+
+  [self removeTrackingRect:tracking_rect];
+  tracking_rect = [self addTrackingRect:[self bounds] owner:self userData:NULL assumeInside:NO];
 }
 
 /* Simply forward all events that reach our view to clutter. */
@@ -514,6 +523,19 @@ clutter_stage_osx_set_accept_focus (ClutterStageWindow *stage_window,
   CLUTTER_OSX_POOL_RELEASE();
 }
 
+static void
+clutter_stage_osx_redraw (ClutterStageWindow *stage_window)
+{
+  ClutterStageOSX *stage_osx = CLUTTER_STAGE_OSX (stage_window);
+
+  CLUTTER_OSX_POOL_ALLOC();
+
+  if (stage_osx->view != NULL)
+    [stage_osx->view setNeedsDisplay: YES];
+
+  CLUTTER_OSX_POOL_RELEASE();
+}
+
 static void
 clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
 {
@@ -529,12 +551,13 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->set_cursor_visible = clutter_stage_osx_set_cursor_visible;
   iface->set_user_resizable = clutter_stage_osx_set_user_resizable;
   iface->set_accept_focus   = clutter_stage_osx_set_accept_focus;
+  iface->redraw             = clutter_stage_osx_redraw;
 }
 
 /*************************************************************************/
 ClutterStageWindow *
-clutter_stage_osx_new (ClutterBackend *backend,
-                       ClutterStage   *wrapper)
+_clutter_stage_osx_new (ClutterBackend *backend,
+                        ClutterStage   *wrapper)
 {
   ClutterStageOSX *self;
 
