@@ -42,17 +42,23 @@
 #include "config.h"
 #endif
 
-#include "../clutter-marshal.h"
 #include "clutter-x11-texture-pixmap.h"
 #include "clutter-x11.h"
 #include "clutter-backend-x11.h"
+
+#include "clutter-actor-private.h"
+#include "clutter-marshal.h"
+#include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
 
 #include "cogl/cogl.h"
 #include "cogl/winsys/cogl-texture-pixmap-x11.h"
 
 #include <X11/extensions/Xdamage.h>
+
+#if HAVE_XCOMPOSITE
 #include <X11/extensions/Xcomposite.h>
+#endif
 
 enum
 {
@@ -89,6 +95,13 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
                                              gint                     width,
                                              gint                     height);
 static void
+clutter_x11_texture_pixmap_sync_window_internal (ClutterX11TexturePixmap *texture,
+                                                 int                      x,
+                                                 int                      y,
+                                                 int                      width,
+                                                 int                      height,
+                                                 gboolean                 override_redirect);
+static void
 clutter_x11_texture_pixmap_set_mapped (ClutterX11TexturePixmap *texture, gboolean mapped);
 static void
 clutter_x11_texture_pixmap_destroyed (ClutterX11TexturePixmap *texture);
@@ -105,8 +118,11 @@ struct _ClutterX11TexturePixmapPrivate
   Damage        damage;
 
   gint          window_x, window_y;
+  gint          window_width, window_height;
 
   guint window_redirect_automatic : 1;
+  /* FIXME: this is inconsistently either whether the window is mapped or whether
+   * it is viewable, and isn't updated correctly. */
   guint window_mapped             : 1;
   guint destroyed                 : 1;
   guint owns_pixmap               : 1;
@@ -206,8 +222,20 @@ on_x_event_filter_too (XEvent *xev, ClutterEvent *cev, gpointer data)
 
   switch (xev->type) {
   case MapNotify:
+    clutter_x11_texture_pixmap_sync_window_internal (texture,
+                                                     priv->window_x,
+                                                     priv->window_y,
+                                                     priv->window_width,
+                                                     priv->window_height,
+                                                     priv->override_redirect);
+    break;
   case ConfigureNotify:
-    clutter_x11_texture_pixmap_sync_window (texture);
+    clutter_x11_texture_pixmap_sync_window_internal (texture,
+                                                     xev->xconfigure.x,
+                                                     xev->xconfigure.y,
+                                                     xev->xconfigure.width,
+                                                     xev->xconfigure.height,
+                                                     xev->xconfigure.override_redirect);
     break;
   case UnmapNotify:
     clutter_x11_texture_pixmap_set_mapped (texture, FALSE);
@@ -304,6 +332,13 @@ free_damage_resources (ClutterX11TexturePixmap *texture)
     }
 }
 
+static gboolean
+clutter_x11_texture_pixmap_get_paint_volume (ClutterActor       *self,
+                                             ClutterPaintVolume *volume)
+{
+  return clutter_paint_volume_set_from_allocation (volume, self);
+}
+
 static void
 clutter_x11_texture_pixmap_real_queue_damage_redraw (
                                               ClutterX11TexturePixmap *texture,
@@ -317,7 +352,8 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
   ClutterActorBox  allocation;
   float            scale_x;
   float            scale_y;
-  ClutterActorBox  clip;
+  ClutterVertex    origin;
+  ClutterPaintVolume clip;
 
   /* NB: clutter_actor_queue_clipped_redraw expects a box in the actor's
    * coordinate space so we need to convert from pixmap coordinates to
@@ -344,14 +380,17 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
   scale_x = (allocation.x2 - allocation.x1) / priv->pixmap_width;
   scale_y = (allocation.y2 - allocation.y1) / priv->pixmap_height;
 
-  clip.x1 = x * scale_x;
-  clip.y1 = y * scale_y;
-  clip.x2 =  clip.x1 + width * scale_x;
-  clip.y2 = clip.y1 + height * scale_y;
+  _clutter_paint_volume_init_static (self, &clip);
 
-  _clutter_actor_queue_redraw_with_clip (self,
-                                         CLUTTER_REDRAW_CLIPPED_TO_BOX,
-                                         &clip);
+  origin.x = x * scale_x;
+  origin.y = y * scale_y;
+  origin.z = 0;
+  clutter_paint_volume_set_origin (&clip, &origin);
+  clutter_paint_volume_set_width (&clip, width * scale_x);
+  clutter_paint_volume_set_height (&clip, height * scale_y);
+
+  _clutter_actor_queue_redraw_with_clip (self, 0, &clip);
+  clutter_paint_volume_free (&clip);
 }
 
 static void
@@ -495,11 +534,14 @@ static void
 clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 {
   GObjectClass      *object_class = G_OBJECT_CLASS (klass);
+  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   GParamSpec        *pspec;
   ClutterBackend    *default_backend;
 
   g_type_class_add_private (klass, sizeof (ClutterX11TexturePixmapPrivate));
 
+  actor_class->get_paint_volume = clutter_x11_texture_pixmap_get_paint_volume;
+
   object_class->dispose      = clutter_x11_texture_pixmap_dispose;
   object_class->set_property = clutter_x11_texture_pixmap_set_property;
   object_class->get_property = clutter_x11_texture_pixmap_get_property;
@@ -621,7 +663,7 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
    * ClutterX11TexturePixmap::update-area:
    * @texture: the object which received the signal
    *
-   * The ::hide signal is emitted to ask the texture to update its
+   * The ::update-area signal is emitted to ask the texture to update its
    * content from its source pixmap.
    *
    * Since: 0.8
@@ -906,7 +948,9 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
  * clutter_x11_texture_pixmap_set_pixmap().
  *
  * If you want to display a window in a #ClutterTexture, you probably want
- * this function, or its older sister, clutter_glx_texture_pixmap_set_window()
+ * this function, or its older sister, clutter_glx_texture_pixmap_set_window().
+ *
+ * This function has no effect unless the XComposite extension is available.
  *
  * Since: 0.8
  */
@@ -921,11 +965,12 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
 
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
-  priv = texture->priv;
-
-  if (!clutter_x11_has_composite_extension())
+  if (!clutter_x11_has_composite_extension ())
     return;
 
+#if HAVE_XCOMPOSITE
+  priv = texture->priv;
+
   if (priv->window == window && automatic == priv->window_redirect_automatic)
     return;
 
@@ -972,12 +1017,9 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
 
   clutter_x11_untrap_x_errors ();
 
-  if (priv->window)
-    {
-      XSelectInput (dpy, priv->window,
-                    attr.your_event_mask | StructureNotifyMask);
-      clutter_x11_add_filter (on_x_event_filter_too, (gpointer)texture);
-    }
+  XSelectInput (dpy, priv->window,
+                attr.your_event_mask | StructureNotifyMask);
+  clutter_x11_add_filter (on_x_event_filter_too, (gpointer)texture);
 
   g_object_ref (texture);
   g_object_notify (G_OBJECT (texture), "window");
@@ -985,7 +1027,110 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
   clutter_x11_texture_pixmap_set_mapped (texture,
                                          attr.map_state == IsViewable);
 
-  clutter_x11_texture_pixmap_sync_window (texture);
+  clutter_x11_texture_pixmap_sync_window_internal (texture,
+                                                   attr.x, attr.y,
+                                                   attr.width, attr.height,
+                                                   attr.override_redirect);
+  g_object_unref (texture);
+
+#endif /* HAVE_XCOMPOSITE */
+}
+
+static void
+clutter_x11_texture_pixmap_sync_window_internal (ClutterX11TexturePixmap *texture,
+                                                 int                      x,
+                                                 int                      y,
+                                                 int                      width,
+                                                 int                      height,
+                                                 gboolean                 override_redirect)
+{
+  ClutterX11TexturePixmapPrivate *priv;
+  Pixmap pixmap = None;
+  gboolean mapped = FALSE;
+  gboolean notify_x = FALSE;
+  gboolean notify_y = FALSE;
+  gboolean notify_override_redirect = FALSE;
+
+  priv = texture->priv;
+
+  if (priv->destroyed)
+    return;
+
+  notify_x = x != priv->window_x;
+  notify_y = y != priv->window_y;
+  notify_override_redirect = override_redirect != priv->override_redirect;
+  priv->window_x = x;
+  priv->window_y = y;
+  priv->window_width = width;
+  priv->window_height = height;
+  priv->override_redirect = override_redirect;
+
+  if (!clutter_x11_has_composite_extension())
+    {
+      /* FIXME: this should just be an error, this is unlikely to work worth anything */
+      clutter_x11_texture_pixmap_set_pixmap (texture, priv->window);
+      return;
+    }
+
+  if (priv->pixmap == None || width != priv->pixmap_width || height != priv->pixmap_height)
+    {
+      /* Note that we're checking the size from the event against the size we obtained
+       * from the last XCompositeNameWindowPixmap/XGetGeometry pair. This will always
+       * end up right in the end, but we can have a series like:
+       *
+       *  Window sized to 100x100
+       *  Window sized to 110x110
+       *  Window sized to 120x120
+       *                          Configure received for 100x100 - NameWindowPixmap
+       *                          Configure received for 110x110 - NameWindowPixmap
+       *                          Configure received for 120x120 - last size OK
+       *
+       * Where we NameWindowPixmap several times in a row. (Using pixmap_width/pixmap_height
+       * rather than window_width/window_height saves the last one.)
+       */
+
+      Display *dpy = clutter_x11_get_default_display ();
+
+      /* NB: It's only valid to name a pixmap if the window is viewable.
+       *
+       * We don't explicitly check this though since there would be a race
+       * between checking and naming unless we use a server grab which is
+       * undesireable.
+       *
+       * Instead we gracefully handle any error with naming the pixmap.
+       */
+      clutter_x11_trap_x_errors ();
+
+      pixmap = XCompositeNameWindowPixmap (dpy, priv->window);
+
+      /* Possible improvement: combine with the XGetGeometry in
+       * clutter_x11_texture_pixmap_set_pixmap() */
+      XSync(dpy, False);
+
+      if (clutter_x11_untrap_x_errors ())
+	pixmap = None;
+    }
+
+  g_object_ref (texture); /* guard against unparent */
+  g_object_freeze_notify (G_OBJECT (texture));
+
+  /* FIXME: mapped is always FALSE */
+  clutter_x11_texture_pixmap_set_mapped (texture, mapped);
+
+  if (pixmap)
+    {
+      clutter_x11_texture_pixmap_set_pixmap (texture, pixmap);
+      priv->owns_pixmap = TRUE;
+    }
+
+  if (notify_override_redirect)
+    g_object_notify (G_OBJECT (texture), "window-override-redirect");
+  if (notify_x)
+    g_object_notify (G_OBJECT (texture), "window-x");
+  if (notify_y)
+    g_object_notify (G_OBJECT (texture), "window-y");
+
+  g_object_thaw_notify (G_OBJECT (texture));
   g_object_unref (texture);
 }
 
@@ -1002,7 +1147,6 @@ void
 clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
 {
   ClutterX11TexturePixmapPrivate *priv;
-  Pixmap pixmap;
 
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
@@ -1011,66 +1155,22 @@ clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
   if (priv->destroyed)
     return;
 
-  if (!clutter_x11_has_composite_extension())
-    {
-      clutter_x11_texture_pixmap_set_pixmap (texture, priv->window);
-      return;
-    }
-
   if (priv->window)
     {
       XWindowAttributes attr;
       Display *dpy = clutter_x11_get_default_display ();
-      gboolean mapped = FALSE;
-      gboolean notify_x = FALSE;
-      gboolean notify_y = FALSE;
-      gboolean notify_override_redirect = FALSE;
       Status status;
 
-      /* NB: It's only valid to name a pixmap if the window is viewable.
-       *
-       * We don't explicitly check this though since there would be a race
-       * between checking and naming unless we use a server grab which is
-       * undesireable.
-       *
-       * Instead we gracefully handle any error with naming the pixmap.
-       */
       clutter_x11_trap_x_errors ();
 
-      pixmap = XCompositeNameWindowPixmap (dpy, priv->window);
-
       status = XGetWindowAttributes (dpy, priv->window, &attr);
       if (status != 0)
-	{
-	  notify_x = attr.x != priv->window_x;
-	  notify_y = attr.y != priv->window_y;
-	  notify_override_redirect = attr.override_redirect != priv->override_redirect;
-	  priv->window_x = attr.x;
-	  priv->window_y = attr.y;
-	  priv->override_redirect = attr.override_redirect;
-	}
+        clutter_x11_texture_pixmap_sync_window_internal (texture,
+                                                         attr.x, attr.y,
+                                                         attr.width, attr.height,
+                                                         attr.override_redirect);
 
-      /* We rely on XGetWindowAttributes() implicitly syncing with the server
-       * so if there was an error naming the pixmap that will have been
-       * caught by now. */
-      if (clutter_x11_untrap_x_errors ())
-	pixmap = None;
-
-      g_object_ref (texture); /* guard against unparent */
-      if (pixmap)
-        {
-          clutter_x11_texture_pixmap_set_pixmap (texture, pixmap);
-          priv->owns_pixmap = TRUE;
-        }
-      clutter_x11_texture_pixmap_set_mapped (texture, mapped);
-      /* could do more clever things with a signal, i guess.. */
-      if (notify_override_redirect)
-        g_object_notify (G_OBJECT (texture), "window-override-redirect");
-      if (notify_x)
-        g_object_notify (G_OBJECT (texture), "window-x");
-      if (notify_y)
-        g_object_notify (G_OBJECT (texture), "window-y");
-      g_object_unref (texture);
+      clutter_x11_untrap_x_errors ();
     }
 }
 
